% bLobatto Lobatto shape function.
%    P = bLobatto(z,n) computes Lobatto shape functions of degree 1,...,n, 
%    which are linear hat-functions and anti-derivatives of 
%    Legendre polynomials for degree >=2  evaluated for each element
%    of z.  n must be a scalar integer and z must contain complex values. 
%
% Examples: 
%   x = linspace(-1,1,100);
%   fx = bLobatto(x,5);
%   plot(x,fx);
%
% (c) epsBEM team 2012

function value = bLobatto(z,n)

z = z(:);

value = ones(size(z,1),n+1);
value(:,1) = (1-z)/2;
if(n>0)
  value(:,2) = (1+z)/2;
end
if(n>1)
  value(:,3) = (z.^2-1)./2;
end

for k=1:n-2
    value(:,k+3) = ((2*k+1)*z.*value(:,k+2)-(k-1)*value(:,k+1))./(k+2);
end
