#include "mex.h"

#if defined(MATLAB73) 
  #define  intSize    mwSize  
  #define  intIndex   mwIndex 
#else
  #define  intSize   int
  #define  intIndex  int
#endif

void pnpoly(int nvert, double *vertx, double *verty, int ntest, double *testx, double *testy, double *res)
{ /* PNPOLY - Point Inclusion in Polygon Test
   * W. Randolph Franklin (WRF) 
   *
   * Copyright (c) 1970-2003, Wm. Randolph Franklin 
   * Permission is hereby granted, free of charge, to any person obtaining 
   * a copy of this software and associated documentation files 
   * (the "Software"), to deal in the Software without restriction, 
   * including without limitation the rights to use, copy, modify, merge, 
   * publish, distribute, sublicense, and/or sell copies of the Software, 
   * and to permit persons to whom the Software is furnished to do so, 
   * subject to the following conditions: 
   *
   * Redistributions of source code must retain the above copyright notice, 
   * this list of conditions and the following disclaimers. 
   * Redistributions in binary form must reproduce the above copyright 
   * notice in the documentation and/or other materials provided with the 
   * distribution. 
   * The name of W. Randolph Franklin may not be used to endorse or promote 
   * products derived from this Software without specific prior written 
   * permission. 
   */
  int i, j, k;
  int c;
  for (k=0; k<ntest; k++){ 
    c = 0;
    for (i = 0, j = nvert-1; i < nvert; j = i++) {
      if ( ((verty[i]>testy[k]) != (verty[j]>testy[k])) 
           && (testx[k] < (vertx[j]-vertx[i]) * (testy[k]-verty[i]) / (verty[j]-verty[i]) + vertx[i]) )
         c = !c;
    }
    res[k] = (double) c;
  }
}

void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  int i;
  double *y, *vertx, *verty, *testx, *testy;
  intSize mrows[4],ncols[4];
  
  /* Check for proper number of arguments. */
  if(nrhs!=4) {
    mexErrMsgTxt("Four input required.");
  } else if(nlhs>1) {
    mexErrMsgTxt("Too many output arguments.");
  }
  
  /* The input must be a noncomplex scalar double.*/
  for (i=0; i<4; i++){
    mrows[i] = mxGetM(prhs[i]);
    ncols[i] = mxGetN(prhs[i]);
    if( !mxIsDouble(prhs[i]) || mxIsComplex(prhs[i]) ||
        !(ncols[i]==1) ) {
      mexErrMsgTxt("Input must be a noncomplex scalar double.");
    }
  }
  if (mrows[0] != mrows[1]){
      mexErrMsgTxt("Input must be a noncomplex scalar double.");
  }
  if (mrows[2] != mrows[3]){
      mexErrMsgTxt("Input must be a noncomplex scalar double.");
  }
    
  /* Create matrix for the return argument. */
  plhs[0] = mxCreateDoubleMatrix(mrows[2],1, mxREAL);
  
  /* Assign pointers to each input and output. */
  vertx = mxGetPr(prhs[0]);
  verty = mxGetPr(prhs[1]);
  testx = mxGetPr(prhs[2]);
  testy = mxGetPr(prhs[3]);
  y = mxGetPr(plhs[0]);
  
  /* Call the timestwo subroutine. */
  pnpoly((int) mrows[0], vertx, verty, (int) mrows[2], testx, testy, y);
}
